unit CompInstallExistingPackageFrames;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MRUCombos, CompInstallPackageBase, CompInstallTools,
  StdCtrls, Registry;

type
  TInstallExistingPackageFrame = class(TInstallBasePage)
    NameCombo: TMRUComboBox;
    procedure NameComboChange(Sender: TObject);
    procedure FrameEnter(Sender: TObject);
    procedure NameComboEnter(Sender: TObject);
  private
  protected
    function GetPackageName: string; override;
    procedure SetPackageName(const Value: string); override;
  public
    procedure SaveValues(Registry: TRegistry); override;
    procedure LoadValues(Registry: TRegistry); override;
  end;

implementation

uses
  CompInstallConsts;

{$R *.dfm}

{ TInstallExistingPackageFrame }

procedure TInstallExistingPackageFrame.FrameEnter(Sender: TObject);
begin
  inherited;
  NameLabel.FocusControl := NameCombo;
end;

function TInstallExistingPackageFrame.GetPackageName: string;
begin
  Result := NameCombo.Text;
end;

procedure TInstallExistingPackageFrame.LoadValues(Registry: TRegistry);
var
  I, Count: Integer;
  S: string;
begin
  // Why can't TRegistry.ReadInteger work like TRegIniFile.ReadInteger,
  // and return a default value, instead of an exception?
  try
    Count := Registry.ReadInteger(SPackages);
  except
    Count := 0;
  end;
  for I := 0 to Count - 1 do
  begin
    S := Registry.ReadString(SPackageN + IntToStr(I));
    NameCombo.Items.Add(S);
  end;
end;

procedure TInstallExistingPackageFrame.NameComboChange(
  Sender: TObject);
begin
  inherited;
  FindPackageDescription;
end;

procedure TInstallExistingPackageFrame.NameComboEnter(
  Sender: TObject);
var
  DclUsr: string;
begin
  inherited;
  DclUsr := ResolveBDSPath(SDefaultDclUsr);
  if FileExists(DclUsr) then
    NameCombo.Items.Add(DclUsr);
end;

procedure TInstallExistingPackageFrame.SaveValues(Registry: TRegistry);
var
  I: Integer;
begin
  inherited;
  Registry.WriteInteger(SPackages, NameCombo.Items.Count);
  for I := 0 to NameCombo.Items.Count - 1 do
    Registry.WriteString(SPackageN + IntToStr(I), NameCombo.Items[I]);
end;

procedure TInstallExistingPackageFrame.SetPackageName(const Value: string);
begin
  inherited;
  NameCombo.Items.Add(Value);
  NameCombo.ItemIndex := NameCombo.Items.IndexOf(Value);
end;

end.
