unit CompInstallPackageBase;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, StdCtrls, CompInstallTools, Registry;

type
  TInstallBasePage = class(TFrame)
    SearchPathEdit: TEdit;
    DescriptionEdit: TEdit;
    BrowseButton: TButton;
    SearchPathLabel: TLabel;
    NameLabel: TLabel;
    DescriptionLabel: TLabel;
    OpenDialog: TOpenDialog;
    procedure DescriptionEditChange(Sender: TObject);
    procedure BrowseButtonClick(Sender: TObject);
  private
    FDescription: string;
    FSearchPath: string;
    function GetDescription: string;
    function GetSearchPath: string;
    procedure FindDescription(Lines: TStringList);
  protected
    procedure SetPackageName(const Value: string); virtual; abstract;
    function GetPackageName: string; virtual; abstract;
    procedure FindPackageDescription;
  public
    constructor Create(AOwner: TComponent); override;
    procedure LoadValues(Registry: TRegistry); virtual;
    procedure SaveValues(Registry: TRegistry); virtual;
    property PackageName: string read GetPackageName write SetPackageName;
    property Description: string read GetDescription;
    property SearchPath: string read GetSearchPath;
  end;

implementation

{$R *.dfm}

uses
  ToolsAPI,
  StrUtils,
  CompInstallConsts;

constructor TInstallBasePage.Create(AOwner: TComponent);
begin
  inherited;
  FSearchPath := GetSearchPath;
  SearchPathEdit.Text := FSearchPath;
end;

procedure TInstallBasePage.FindDescription(Lines: TStringList);
var
  P2: Integer;
  P1: Integer;
  Line: string;
begin
  for Line in Lines do
  begin
    P1 := AnsiPos(SDescriptionDirective, AnsiUpperCase(Line));
    if P1 > 0 then
    begin
      P1 := P1 + Length(SDescriptionDirective);
      P2 := Length(Line);
      repeat
        Dec(P2);
      until (Line[P2] = '''') and (Line[P2 + 1] = '}');
      DescriptionEdit.Text := Copy(Line, P1, P2 - P1);
      Break;
    end;
  end;
end;

procedure TInstallBasePage.FindPackageDescription;
var
  Package: string;
  Lines: TStringList;
begin
  Package := GetPackageName;
  if not FileExists(Package) then
    DescriptionEdit.Text := ''
  else
  begin
    Lines := TStringList.Create;
    try
      Lines.LoadFromFile(Package);
      FindDescription(Lines);
    finally
      Lines.Free;
    end;
  end;
end;

function TInstallBasePage.GetDescription: string;
begin
  Result := DescriptionEdit.Text;
end;

function TInstallBasePage.GetSearchPath: string;
var
  AServices: IOTAServices;
{$IF RTLVersion < 18.0}
  ARegistry: TRegistry;
{$ELSE}
  AOptions: IOTAEnvironmentOptions;
{$IFEND}
begin
   if not Supports(BorlandIDEServices, IOTAServices, AServices) then
     raise Exception.Create(SNoIDEServices);
{$IF RTLVersion < 18.0}
   begin // Delphi 2005's OTAPI might return wrong version of search path.
     ARegistry := TRegistry.Create(KEY_READ);
     try
       ARegistry.RootKey := HKEY_CURRENT_USER;
       if not ARegistry.OpenKey(AServices.GetBaseRegistryKey
                + '\Library', // do not localize!
                False) then
         raise Exception.Create(StrUnableToOpenKey);
       Result := ARegistry.ReadString('Search Path'); // do not localize!
     finally
       ARegistry.Free;
     end;
   end
{$ELSE}
   begin
     AOptions := AServices.GetEnvironmentOptions;
     Result := AOptions.Values['LibraryPath']; // do not localize!
   end;
{$IFEND}
end;

procedure TInstallBasePage.LoadValues(Registry: TRegistry);
begin
  OpenDialog.InitialDir := Registry.ReadString(SPackageSourceDir);
end;

procedure TInstallBasePage.SaveValues(Registry: TRegistry);
begin
  Registry.WriteString(SPackageSourceDir,
    ExtractFilePath(ExpandFileName(OpenDialog.FileName)));
end;

procedure TInstallBasePage.BrowseButtonClick(Sender: TObject);
var
  DclUsr: string;
begin
  DclUsr := ResolveBDSPath(SDefaultDclUsr);
  if (OpenDialog.Filename = '') and FileExists(DclUsr) then
    OpenDialog.FileName := DclUsr;
  if OpenDialog.Execute then
  begin
    SetPackageName(OpenDialog.FileName);
    FindPackageDescription;
  end;
end;

procedure TInstallBasePage.DescriptionEditChange(Sender: TObject);
begin
  FDescription := DescriptionEdit.Text;
end;

end.
