unit CompInstallWizardDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, StdCtrls, CompInstallPackageBase,
  CompInstallExistingPackageFrames, CompInstallNewPackageFrames,
  CompInstallUnitsFrames, CompInstallTools, Registry;

type
  TInstallWizardDialog = class(TForm)
    PropertySheets: TPageControl;
    ButtonPanel: TPanel;
    BottomBevel: TBevel;
    TargetSelectionPage: TTabSheet;
    ExistingPackagePage: TTabSheet;
    NewPackagePage: TTabSheet;
    CaptionPanel: TPanel;
    TopBevel: TBevel;
    HelpButton: TButton;
    CancelButton: TButton;
    FinishButton: TButton;
    NextButton: TButton;
    BackButton: TButton;
    CaptionLabel: TLabel;
    ExplanationLabel: TLabel;
    WizardImage: TImage;
    StartFrame: TInstallUnitsFrame;
    NewPackageFrame: TInstallNewPackageFrame;
    ExistingPackageFrame: TInstallExistingPackageFrame;
    procedure NewPackageFrameBrowseButtonClick(Sender: TObject);
    procedure ExistingPackageFrameBrowseButtonClick(
      Sender: TObject);
    procedure ExistingPackageFrameNameComboChange(
      Sender: TObject);
    procedure NewPackageFrameNameEditChange(Sender: TObject);
    procedure StartFrameBrowseButtonClick(Sender: TObject);
    procedure StartFrameFileNamesEditChange(Sender: TObject);
    procedure FinishButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BackButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure PackageRadioClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    FExistingPackage: Boolean;
    FUnits: TStrings;
    FPackageName: string;
    FDescription: string;
    FInstallPage: TInstallBasePage;
    FMyResult: Boolean;
    FModule: THandle;
    FPersonality: TPackagePersonality;
    procedure UpdateWizard;
    function CanFinish: Boolean;
    procedure SetProperties;
    procedure LoadValues;
    procedure SaveValues;
    procedure OpenRegistryKey(ARegistry: TRegistry);
  public
    function Execute: Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ExistingPackage: Boolean read FExistingPackage;
    property Units: TStrings read FUnits;
    property InstallPage: TInstallBasePage read FInstallPage;
    property Personality: TPackagePersonality read FPersonality;
    property MyResult: Boolean read FMyResult;
  end;

implementation

{$R *.dfm}

uses
  ToolsAPI, CompInstallConsts;

type
  PPropertySheet = ^TPropertySheet;
  TPropertySheet = record
    Caption: string;
    Explanation: string;
    Bitmap: string;
  end;

resourcestring
  StrSelectCaption       = 'Select units';
  StrSelectExplanation   = 'Select the unit files you want to install, and ' +
                           'whether they must be installed into a new ' +
                           'or an existing package.';
  StrNewPakCaption       = 'Install into new package';
  StrNewPakExplanation   = 'Select the package and enter the description ' +
                           'for the package';
  StrExistPakCaption     = 'Install into existing package';
  StrExistPakExplanation = 'Select the package and change or enter the ' +
                           'description for the package';
  StrInternalError       = 'Component install: Internal error';

const
  pseSelect   = 0;
  pseNew      = 1;
  pseExisting = 2;

  CPropertySheets: array[pseSelect..pseExisting] of TPropertySheet = (
    (Caption: StrSelectCaption;
     Explanation: StrSelectExplanation;
     Bitmap: 'WIZARDANCESTORWIN'), // Do not localize!
    (Caption: StrNewPakCaption;
     Explanation: StrNewPakExplanation;
     Bitmap: 'WIZARDNEWPACKAGEWIN'), // Do not localize!
    (Caption: StrExistPakCaption;
     Explanation: StrExistPakExplanation;
     Bitmap: 'WIZARDEXISTINGPACKAGEWIN')); // Do not localize!

procedure TInstallWizardDialog.BackButtonClick(Sender: TObject);
begin
  case PropertySheets.ActivePageIndex of
    pseNew, pseExisting:
      PropertySheets.ActivePageIndex := pseSelect;
  end;
  UpdateWizard;
end;

procedure TInstallWizardDialog.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

function TInstallWizardDialog.CanFinish: Boolean;
begin
  case PropertySheets.ActivePageIndex of
    pseNew:
      Result := NewPackageFrame.NameEdit.Text <> '';
    pseExisting:
      Result := ExistingPackageFrame.NameCombo.Text <> '';
  else
    Result := False;
  end;
end;

constructor TInstallWizardDialog.Create(AOwner: TComponent);
begin
  inherited;
  FUnits := TStringList.Create;
end;

destructor TInstallWizardDialog.Destroy;
begin
  FUnits.Free;
  inherited;
end;

procedure TInstallWizardDialog.OpenRegistryKey(ARegistry: TRegistry);
var
  AServices: IOTAServices;
  Key: string;
begin
  if not Supports(BorlandIDEServices, IOTAServices, AServices) then
    raise Exception.Create(SNoIDEServices);
  Key := AServices.GetBaseRegistryKey + '\CompInstaller'; // do not localize!
  ARegistry.RootKey := HKEY_CURRENT_USER;
  if not ARegistry.OpenKey(Key, False) then
    ARegistry.CreateKey(Key);
end;

procedure TInstallWizardDialog.LoadValues;
var
  ARegistry: TRegistry;
begin
  ARegistry := TRegistry.Create(KEY_ALL_ACCESS);
  try
    OpenRegistryKey(ARegistry);
    StartFrame.LoadValues(ARegistry);
    NewPackageFrame.LoadValues(ARegistry);
    ExistingPackageFrame.Loadvalues(ARegistry);
  finally
    ARegistry.Free;
  end;
end;

procedure TInstallWizardDialog.SaveValues;
var
  ARegistry: TRegistry;
begin
  ARegistry := TRegistry.Create(KEY_ALL_ACCESS);
  try
    OpenRegistryKey(ARegistry);
    StartFrame.SaveValues(ARegistry);
    NewPackageFrame.SaveValues(ARegistry);
    ExistingPackageFrame.SaveValues(ARegistry);
  finally
    ARegistry.Free;
  end;
end;

function TInstallWizardDialog.Execute: Boolean;
begin
  LoadValues;
  if FModule <> 0 then
    FreeLibrary(FModule);
  ShowModal;
  Result := FMyResult;
  SaveValues;
end;

procedure TInstallWizardDialog.SetProperties;
var
  Frame: TInstallBasePage;
begin
  case PropertySheets.ActivePageIndex of
    pseNew:
      Frame := NewPackageFrame;
    pseExisting:
      Frame := ExistingPackageFrame;
    else
      Frame := nil;
  end;
  FDescription := Frame.Description;
  FPackageName := Frame.PackageName;
  FPersonality := NewPackageFrame.Personality;
  FUnits.Assign(StartFrame.Units);
end;

procedure TInstallWizardDialog.PackageRadioClick(Sender: TObject);
begin
  FExistingPackage := StartFrame.ExistingPackageRadio.Checked;
end;

procedure TInstallWizardDialog.FinishButtonClick(Sender: TObject);
begin
  Close;
  SetProperties;
  case PropertySheets.ActivePageIndex of
    pseNew:
      FInstallPage := NewPackageFrame;
    pseExisting:
      FInstallPage := ExistingPackageFrame;
    else
      raise Exception.Create(StrInternalError);
  end;
  ModalResult := mrOK;
  FMyResult := True;
  FExistingPackage := StartFrame.ExistingPackageRadio.Checked;
end;

procedure TInstallWizardDialog.FormCreate(Sender: TObject);
begin
  PropertySheets.ActivePageIndex := 0;
{$IF (RTLVersion <= 18.0) and not defined(VER185)}
  NewPackageFrame.CppBuilderPackageRadio.Enabled := False;
  NewPackageFrame.DelphiPackageRadio.Checked := True;
{$IFEND}
  UpdateWizard;
end;

procedure TInstallWizardDialog.ExistingPackageFrameBrowseButtonClick(
  Sender: TObject);
begin
  ExistingPackageFrame.BrowseButtonClick(Sender);
  UpdateWizard;
end;

procedure TInstallWizardDialog.ExistingPackageFrameNameComboChange(
  Sender: TObject);
begin
  ExistingPackageFrame.NameComboChange(Sender);
  UpdateWizard;
end;

procedure TInstallWizardDialog.NewPackageFrameBrowseButtonClick(
  Sender: TObject);
begin
  NewPackageFrame.BrowseButtonClick(Sender);
  UpdateWizard;
end;

procedure TInstallWizardDialog.NewPackageFrameNameEditChange(
  Sender: TObject);
begin
  NewPackageFrame.NameEditChange(Sender);
  UpdateWizard;
end;

procedure TInstallWizardDialog.StartFrameFileNamesEditChange(
  Sender: TObject);
begin
  UpdateWizard;
end;

procedure TInstallWizardDialog.StartFrameBrowseButtonClick(
  Sender: TObject);
begin
  StartFrame.UnitsBrowseButtonClick(Sender);
  UpdateWizard;
end;

procedure TInstallWizardDialog.UpdateWizard;
var
  P: PPropertySheet;
begin

  if FModule = 0 then
    FModule := LoadLibrary(PChar(BinDirectory + SDelphiCoreIDE));
  P := @CPropertySheets[PropertySheets.ActivePageIndex];
  CaptionLabel.Caption := P^.Caption;
  ExplanationLabel.Caption := P^.Explanation;
  WizardImage.Picture.Bitmap.LoadFromResourceName(FModule, P^.Bitmap);
  BackButton.Enabled := PropertySheets.ActivePageIndex > pseSelect;
  NextButton.Enabled := (PropertySheets.ActivePageIndex = pseSelect) and
                        (StartFrame.UnitFileNamesEdit.Text <> '');
  FinishButton.Enabled := CanFinish;
end;

procedure TInstallWizardDialog.NextButtonClick(Sender: TObject);
begin
  case PropertySheets.ActivePageIndex of
    pseSelect:
      if StartFrame.ExistingPackageRadio.Checked then
        PropertySheets.ActivePageIndex := pseExisting
      else
        PropertySheets.ActivePageIndex := pseNew;
  end;
  UpdateWizard;
end;

end.
